@@ -1,4 +1,5 @@
 import copy
+import datetime
 import os
 import re
 
@@ -8,10 +9,11 @@ import google.cloud.bigquery
 from google.cloud.bigquery.dbapi import Cursor
 from googleauthentication import GoogleAuthentication
 
-from bigquery.core.Column import change_columns_type, columns_type_bool_to_str, change_column_value_to_string
+from bigquery.core.Column import change_columns_type, columns_type_bool_to_str, change_column_value_to_string, \
+    change_column_value_to_int, change_column_value_to_float
 from bigquery.core.tools.print_colors import C
 from bigquery.core.Table import create_table, create_columns
-
+import logging
 
 class BigQueryDBStream(dbstream.DBStream):
     def __init__(self, instance_name, client_id, google_auth: GoogleAuthentication):
@@ -89,6 +91,16 @@ class BigQueryDBStream(dbstream.DBStream):
             final_data = []
             for x in temp_row:
                 for y in x:
+                    if not isinstance(y, bool) \
+                            and not isinstance(y, datetime.date) \
+                            and not isinstance(y, datetime.datetime):
+                        try:
+                            y = int(y)
+                        except:
+                            try:
+                                y = float(y)
+                            except:
+                                pass
                     final_data.append(y)
 
             temp_string = ','.join(map(lambda a: '(' + ','.join(map(lambda b: '%s', a)) + ')', tuple(temp_row)))
@@ -117,7 +129,8 @@ class BigQueryDBStream(dbstream.DBStream):
                           data,
                           replace=True,
                           batch_size=1000,
-                          other_table_to_update=None
+                          other_table_to_update=None,
+                          n=1
                           ):
         """
         data = {
@@ -130,29 +143,50 @@ class BigQueryDBStream(dbstream.DBStream):
         try:
             self._send(data, replace=replace, batch_size=batch_size)
         except Exception as e:
-            if ("value has type float64 which cannot be inserted into" in str(e).lower()
-                or "value has type int64 which cannot be inserted into" in str(e).lower()
-                or "value has type bool which cannot be inserted into" in str(e).lower()) \
-                    and "string" in str(e).lower():
+            error_lowercase = str(e).lower()
+            logging.log(error_lowercase.split("\n")[0])
+            if ("value has type float64 which cannot be inserted into" in error_lowercase
+                or "value has type int64 which cannot be inserted into" in error_lowercase
+                or "value has type bool which cannot be inserted into" in error_lowercase) \
+                    and "string" in error_lowercase:
                 column = str(e).split("column ")[1].split(",")[0]
                 change_column_value_to_string(
                     data=data_copy,
                     column=column,
                 )
-            elif ("value has type float64 which cannot be inserted into" in str(e).lower() and not "string" in str(e).lower())\
-                    or ("value has type string which cannot be inserted into" in str(e).lower() and not "bool" in str(e).lower()):
-                change_columns_type(
-                    self,
-                    data=data_copy,
-                    other_table_to_update=other_table_to_update
-                )
-            elif "value has type string which cannot be inserted into" in str(e).lower() and "bool" in str(e).lower():
+            elif (
+                    "value has type float64 which cannot be inserted into" in error_lowercase and not "string" in error_lowercase) \
+                    or (
+                    "value has type string which cannot be inserted into" in error_lowercase and not "bool" in error_lowercase):
+                column = str(e).split("column ")[1].split(",")[0]
+                if n == 2:
+                    change_columns_type(
+                        self,
+                        data=data_copy,
+                        other_table_to_update=other_table_to_update
+                    )
+                    n = 1
+                else:
+                    if "which has type int" in error_lowercase:
+                        change_column_value_to_int(
+                            data=data_copy,
+                            column=column,
+                        )
+                    elif "which has type float" in error_lowercase:
+                        change_column_value_to_float(
+                            data=data_copy,
+                            column=column,
+                        )
+                    n = 2
+
+            elif "value has type string which cannot be inserted into" in error_lowercase and "bool" in error_lowercase:
                 columns_type_bool_to_str(
                     self,
                     data=data_copy,
                     other_table_to_update=other_table_to_update
                 )
-            elif " was not found " in str(e).lower() and (" table " in str(e).lower() or " dataset " in str(e).lower()):
+            elif " was not found " in error_lowercase and (
+                    " table " in error_lowercase or " dataset " in error_lowercase):
                 print("Destination table doesn't exist! Will be created")
                 create_table(
                     self,
@@ -160,7 +194,7 @@ class BigQueryDBStream(dbstream.DBStream):
                     other_table_to_update=other_table_to_update
                 )
                 replace = False
-            elif " is not present in table " in str(e).lower() and "column" in str(e).lower():
+            elif " is not present in table " in error_lowercase and "column" in error_lowercase:
                 create_columns(
                     self,
                     data=data_copy,
@@ -170,7 +204,7 @@ class BigQueryDBStream(dbstream.DBStream):
                 raise e
 
             self._send_data_custom(data_copy, replace=replace, batch_size=batch_size,
-                                   other_table_to_update=other_table_to_update)
+                                   other_table_to_update=other_table_to_update, n=n)
 
     def clean(self, selecting_id, schema_prefix, table):
         print('trying to clean table %s.%s using %s' % (schema_prefix, table, selecting_id))
@@ -187,6 +221,7 @@ class BigQueryDBStream(dbstream.DBStream):
 
     def get_max(self, schema, table, field, filter_clause=""):
         try:
+            print("SELECT max(%s) as max FROM %s.%s %s" % (field, schema, table, filter_clause))
             r = self.execute_query("SELECT max(%s) as max FROM %s.%s %s" % (field, schema, table, filter_clause))
             return r[0]["max"]
         except Exception as e:
